@@ -16,6 +16,7 @@
 #include "llvm/Module.h"
 #include "llvm/ADT/STLExtras.h"
 #include "llvm/ADT/StringExtras.h"
+#include "llvm/ADT/StringMap.h"
 using namespace llvm;
 
 static char HexDigit(int V) {
@@ -138,56 +139,28 @@ std::string Mangler::getValueName(const GlobalValue *GV, const char * Suffix) {
   // - If V is an intrinsic function, do not change name at all
   // - Otherwise, mangling occurs if global collides with existing name.
   if (isa<Function>(GV) && cast<Function>(GV)->isIntrinsic()) {
-    Name = GV->getName(); // Is an intrinsic function
+    Name = GV->getNameStart(); // Is an intrinsic function
   } else if (!GV->hasName()) {
     // Must mangle the global into a unique ID.
     unsigned TypeUniqueID = getTypeID(GV->getType());
     static unsigned GlobalID = 0;
     Name = "__unnamed_" + utostr(TypeUniqueID) + "_" + utostr(GlobalID++);
-  } else if (!MangledGlobals.count(GV)) {
-    Name = makeNameProper(GV->getName() + Suffix, Prefix);
   } else {
-    unsigned TypeUniqueID = getTypeID(GV->getType());
-    Name = "l" + utostr(TypeUniqueID) + "_" + makeNameProper(GV->getName());
+    Name = makeNameProper(GV->getName() + Suffix, Prefix);
+    std::string prefix;
+    if (GV->hasPrivateLinkage())
+      prefix = PrivatePrefix;
+    else
+      prefix = "";
+    Name = prefix + Name;
   }
 
   return Name;
 }
 
-void Mangler::InsertName(GlobalValue *GV,
-                         std::map<std::string, GlobalValue*> &Names) {
-  if (!GV->hasName())   // We must mangle unnamed globals.
-    return;
-
-  // Figure out if this is already used.
-  GlobalValue *&ExistingValue = Names[GV->getName()];
-  if (!ExistingValue) {
-    ExistingValue = GV;
-  } else {
-    // If GV is external but the existing one is static, mangle the existing one
-    if ((GV->hasExternalLinkage() || GV->hasDLLImportLinkage()) &&
-        !(ExistingValue->hasExternalLinkage() || ExistingValue->hasDLLImportLinkage())) {
-      MangledGlobals.insert(ExistingValue);
-      ExistingValue = GV;
-    } else if ((GV->hasExternalLinkage() ||
-                GV->hasDLLImportLinkage()) &&
-               (ExistingValue->hasExternalLinkage() ||
-                ExistingValue->hasDLLImportLinkage()) &&
-               GV->isDeclaration() &&
-               ExistingValue->isDeclaration()) {
-      // If the two globals both have external inkage, and are both external,
-      // don't mangle either of them, we just have some silly type mismatch.
-    } else {
-      // Otherwise, mangle GV
-      MangledGlobals.insert(GV);
-    }
-  }
-}
-
-
-Mangler::Mangler(Module &M, const char *prefix)
-  : Prefix(prefix), UseQuotes(false), PreserveAsmNames(false),
-    Count(0), TypeCounter(0) {
+Mangler::Mangler(Module &M, const char *prefix, const char *privatePrefix)
+  : Prefix(prefix), PrivatePrefix (privatePrefix), UseQuotes(false),
+    PreserveAsmNames(false), Count(0), TypeCounter(0) {
   std::fill(AcceptableChars, array_endof(AcceptableChars), 0);
 
   // Letters and numbers are acceptable.
@@ -202,12 +175,4 @@ Mangler::Mangler(Module &M, const char *prefix)
   markCharAcceptable('_');
   markCharAcceptable('$');
   markCharAcceptable('.');
-    
-  // Calculate which global values have names that will collide when we throw
-  // away type information.
-  std::map<std::string, GlobalValue*> Names;
-  for (Module::iterator I = M.begin(), E = M.end(); I != E; ++I)
-    InsertName(I, Names);
-  for (Module::global_iterator I = M.global_begin(), E = M.global_end(); I != E; ++I)
-    InsertName(I, Names);
 }
